@@ -52,6 +52,40 @@ typedef struct nano_dialer_s {
   nng_tls_config *tls;
 } nano_dialer;
 
+typedef struct nano_stream_listener_s {
+  nng_stream_listener *list;
+  nng_tls_config *tls;
+} nano_stream_listener;
+
+typedef struct nano_stream_dialer_s {
+  nng_stream_dialer *dial;
+  nng_tls_config *tls;
+} nano_stream_dialer;
+
+typedef enum nano_aio_typ {
+  SENDAIO,
+  RECVAIO,
+  IOV_SENDAIO,
+  IOV_RECVAIO,
+  HTTP_AIO
+} nano_aio_typ;
+
+typedef struct nano_aio_s {
+  nng_aio *aio;
+  nano_aio_typ type;
+  int mode;
+  int result;
+  void *data;
+} nano_aio;
+
+typedef struct nano_handle_s {
+  nng_url *url;
+  nng_http_client *cli;
+  nng_http_req *req;
+  nng_http_res *res;
+  nng_tls_config *cfg;
+} nano_handle;
+
 typedef struct nano_cv_s {
   int condition;
   uint8_t flag;
@@ -59,6 +93,16 @@ typedef struct nano_cv_s {
   nng_cv *cv;
 } nano_cv;
 
+typedef struct nano_cv_aio_s {
+  nano_cv *cv;
+  nano_aio *aio;
+} nano_cv_aio;
+
+typedef struct nano_cv_duo_s {
+  nano_cv *cv;
+  nano_cv *cv2;
+} nano_cv_duo;
+
 #endif
 
 #ifdef NANONEXT_TIME
@@ -72,6 +116,13 @@ typedef struct nano_cv_s {
 #include <mbedtls/sha256.h>
 #include <mbedtls/sha512.h>
 #include <mbedtls/version.h>
+
+#define SHA1_KEY_SIZE 20
+#define SHA224_KEY_SIZE 28
+#define SHA256_KEY_SIZE 32
+#define SHA384_KEY_SIZE 48
+#define SHA512_KEY_SIZE 64
+
 #endif
 
 #ifdef NANONEXT_KEYCERT
@@ -88,9 +139,6 @@ typedef struct nano_cv_s {
 #include <mbedtls/ctr_drbg.h>
 #include <mbedtls/md.h>
 #include <mbedtls/error.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <errno.h>
 #endif
 
@@ -103,15 +151,38 @@ typedef struct nano_cv_s {
 
 #define ERROR_OUT(xc) Rf_error("%d | %s", xc, nng_strerror(xc))
 #define ERROR_RET(xc) { Rf_warning("%d | %s", xc, nng_strerror(xc)); return mk_error(xc); }
+#define NANO_CHAR(val, len) Rf_mkCharLenCE(val, len, CE_NATIVE)
+#define NANO_STRING(val, len) Rf_ScalarString(NANO_CHAR(val, len))
+#define NANO_ENCODE(buf, data) { SEXP enc = nano_encode(data); NANO_INIT(buf, RAW(enc), XLENGTH(enc)); }
+#define NANONEXT_INIT_BUFSIZE 16384
+#define NANONEXT_SERIAL_VER 3
+#define NANO_ALLOC(x, sz)                                      \
+  x.buf = R_Calloc(sz, unsigned char);                         \
+  x.len = (R_xlen_t) sz;                                       \
+  x.cur = 0;
+#define NANO_INIT(x, ptr, sz)                                  \
+  x.buf = ptr;                                                 \
+  x.len = 0;                                                   \
+  x.cur = (R_xlen_t) sz;
+#define NANO_FREE(x) if (x.len) R_Free(x.buf);
+
+typedef struct nano_buf_s {
+  unsigned char *buf;
+  size_t len;
+  size_t cur;
+} nano_buf;
+
 extern SEXP mk_error(const int);
 extern SEXP mk_error_recv(const int);
 extern SEXP mk_error_ncurl(const int);
 extern SEXP nano_decode(unsigned char *, size_t, const int, const int);
 extern SEXP nano_encode(SEXP);
-extern SEXP nano_encodes(SEXP, SEXP);
+extern int nano_encodes(SEXP);
 extern int nano_matcharg(SEXP);
 extern int nano_matchargs(SEXP);
-extern SEXP rawOneString(unsigned char *, R_xlen_t, R_xlen_t *);
+extern nano_buf nano_serialize(SEXP);
+extern SEXP nano_unserialize(unsigned char *, size_t);
+extern SEXP rawToChar(unsigned char *, size_t);
 extern void socket_finalizer(SEXP);
 extern void dialer_finalizer(SEXP);
 extern void listener_finalizer(SEXP);
@@ -125,22 +196,16 @@ extern SEXP nano_DotcallSymbol;
 extern SEXP nano_HeadersSymbol;
 extern SEXP nano_IdSymbol;
 extern SEXP nano_ListenerSymbol;
-extern SEXP nano_NewEnvSymbol;
-extern SEXP nano_PipeSymbol;
 extern SEXP nano_ProtocolSymbol;
 extern SEXP nano_RawSymbol;
 extern SEXP nano_ResponseSymbol;
 extern SEXP nano_ResultSymbol;
-extern SEXP nano_RtcSymbol;
-extern SEXP nano_SerialSymbol;
-extern SEXP nano_SessionSymbol;
 extern SEXP nano_SocketSymbol;
 extern SEXP nano_StateSymbol;
 extern SEXP nano_StatusSymbol;
 extern SEXP nano_StreamSymbol;
 extern SEXP nano_TextframesSymbol;
 extern SEXP nano_TlsSymbol;
-extern SEXP nano_UnserSymbol;
 extern SEXP nano_UrlSymbol;
 
 extern SEXP nano_addRedirect;
@@ -184,6 +249,7 @@ extern SEXP rnng_dialer_close(SEXP);
 extern SEXP rnng_dialer_start(SEXP, SEXP);
 extern SEXP rnng_fini(void);
 extern SEXP rnng_get_opt(SEXP, SEXP);
+extern SEXP rnng_is_error_value(SEXP);
 extern SEXP rnng_is_nul_byte(SEXP);
 extern SEXP rnng_listen(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_listener_close(SEXP);
@@ -220,7 +286,6 @@ extern SEXP rnng_stream_listen(SEXP, SEXP, SEXP);
 extern SEXP rnng_strcat(SEXP, SEXP);
 extern SEXP rnng_strerror(SEXP);
 extern SEXP rnng_subscribe(SEXP, SEXP, SEXP);
-extern SEXP rnng_timed_signal(SEXP);
 extern SEXP rnng_tls_config(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_unresolved(SEXP);
 extern SEXP rnng_unresolved2(SEXP);
