@@ -62,10 +62,8 @@ static void rnng_messenger_thread(void *args) {
                  tms->tm_year + 1900, tms->tm_mon + 1, tms->tm_mday,
                  tms->tm_hour, tms->tm_min, tms->tm_sec);
         nng_free(buf, sz);
-        const SEXP enc = nano_encode(key);
-        const R_xlen_t xlen = Rf_xlength(enc);
-        unsigned char *dp = RAW(enc);
-        xc = nng_send(*sock, dp, xlen, NNG_FLAG_NONBLOCK);
+        SEXP enc = nano_encode(key);
+        xc = nng_send(*sock, RAW(enc), XLENGTH(enc), NNG_FLAG_NONBLOCK);
         if (xc) {
           REprintf("| messenger session ended: %d-%02d-%02d %02d:%02d:%02d\n",
                    tms->tm_year + 1900, tms->tm_mon + 1, tms->tm_mday,
@@ -161,58 +159,3 @@ SEXP rnng_messenger_thread_create(SEXP list) {
   return socket;
 
 }
-
-static void rnng_timer_thread(void *args) {
-
-  SEXP pairlist = (SEXP) args;
-  SEXP cvar = CADR(pairlist);
-  SEXP time = CADDR(pairlist);
-  SEXP flag = CADDDR(pairlist);
-
-  nano_cv *ncv = R_ExternalPtrAddr(cvar);
-  nng_cv *cv = ncv->cv;
-  nng_mtx *mtx = ncv->mtx;
-
-  switch (TYPEOF(time)) {
-  case INTSXP:
-    nng_msleep((nng_duration) abs(INTEGER(time)[0]));
-    break;
-  case REALSXP:
-    nng_msleep((nng_duration) abs(Rf_asInteger(time)));
-    break;
-  }
-
-  if (Rf_asLogical(flag) == 1) {
-    nng_mtx_lock(mtx);
-    ncv->flag = 1;
-    ncv->condition++;
-    nng_cv_wake(cv);
-    nng_mtx_unlock(mtx);
-  } else {
-    nng_mtx_lock(mtx);
-    ncv->condition++;
-    nng_cv_wake(cv);
-    nng_mtx_unlock(mtx);
-  }
-
-}
-
-SEXP rnng_timed_signal(SEXP args) {
-
-  nng_thread *thr;
-  SEXP cvar, xptr;
-
-  cvar = CADR(args);
-  if (R_ExternalPtrTag(cvar) != nano_CvSymbol)
-    Rf_error("'cv' is not a valid Condition Variable");
-
-  nng_thread_create(&thr, rnng_timer_thread, args);
-
-  PROTECT(xptr = R_MakeExternalPtr(thr, R_NilValue, R_NilValue));
-  R_RegisterCFinalizerEx(xptr, thread_finalizer, TRUE);
-  Rf_classgets(xptr, Rf_mkString("thread"));
-
-  UNPROTECT(1);
-  return xptr;
-
-}
