@@ -2,6 +2,7 @@ library(nanonext)
 nanotest <- function(x) invisible(x || stop("is not TRUE when expected to be TRUE"))
 nanotestw <- function(x) invisible(suppressWarnings(x) || stop("is not TRUE when expected to be TRUE"))
 nanotestz <- function(x) invisible(x == 0L || stop("does not equal 0L as expected"))
+nanotestaio <- function(x) invisible(is_aio(x) || stop("is not of class 'recvAio' or 'sendAio' as expected"))
 nanotestnano <- function(x) invisible(is_nano(x) || stop("does not inherit from class 'nano' as expected"))
 nanotestn <- function(x) invisible(is.null(x) || stop("is not NULL when expected to be NULL"))
 nanotestnn <- function(x) invisible(!is.null(x) || stop("is NULL when expected to be not NULL"))
@@ -11,8 +12,8 @@ nanotesterr <- function(x, e = "")
   invisible(grepl(e, tryCatch(x, error = identity)[["message"]], fixed = TRUE) || stop("expected error message '", e, "' not generated"))
 
 nng_version()
-n <- nano("pair", listen = "inproc://nanonext", autostart = FALSE)
-n1 <- nano("pair", dial = "inproc://nanonext", autostart = FALSE)
+nanotestnano(n <- nano("pair", listen = "inproc://nanonext", autostart = FALSE))
+nanotestnano(n1 <- nano("pair", dial = "inproc://nanonext", autostart = FALSE))
 nanotest(inherits(n, "nanoObject"))
 nanotest(inherits(n$socket, "nanoSocket"))
 nanotest(inherits(n$socket, "nano"))
@@ -64,7 +65,7 @@ nanotest(n1$dialer[[1]]$state == "started")
 nanotesterr(n$send(list(), mode = "raw"), "atomic vector type")
 nanotesterr(n$recv(mode = "none"), "mode")
 nanotesterr(n$recv(mode = "c"), "mode")
-raio <- n$recv_aio(timeout = 1L)
+nanotestaio(raio <- n$recv_aio(timeout = 1L))
 nanotestp(raio)
 nanotest(is_error_value(call_aio(raio)$data))
 nanotest(is_error_value(raio$data))
@@ -74,30 +75,28 @@ nanotestz(n1$send("test", mode = "raw", block = 500))
 nanotest(n$recv("character", block = 500, keep.raw = TRUE)$data == "test")
 nanotestz(n1$send(c(1L, 0L, 0L), mode = "r"))
 nanotest(is.list(n$recv("int", keep.raw = TRUE)))
-saio <- n$send_aio(data.frame(), timeout = 500)
+nanotestaio(saio <- n$send_aio(data.frame(random(1e4L)), timeout = 900))
 nanotestp(saio)
-nanotest(is_aio(call_aio(saio)))
+nanotestaio(call_aio(saio))
 nanotest(!is_nano(saio))
 nanotest(is.integer(saio$result))
 nanotesterr(n$send("wrong mode", mode = "none"), "mode")
-raio <- n1$recv_aio(timeout = 500, keep.raw = TRUE)
+nanotestaio(raio <- n1$recv_aio(timeout = 500, keep.raw = TRUE))
 nanotestp(raio)
 raio$newfield <- "doesnotwork"
 raio[["newfield"]] <- "doesnotwork"
 nanotestn(raio$newfield)
-nanotest(is_aio(raio))
-saio <- n$send_aio(c(1.1, 2.2), mode = "raw", timeout = 500)
+nanotestaio(saio <- n$send_aio(c(1.1, 2.2), mode = "raw", timeout = 500))
 saio$newfield <- "doesnotwork"
 saio[["newfield"]] <- "doesnotwork"
 nanotestn(saio$newfield)
-nanotest(is_aio(saio))
 nanotest(is.logical(.unresolved(saio)))
 nanotest(is.logical(unresolved(saio)))
 nanotestnn(raio$raw)
 nanotest(identical(unserialize(call_aio(raio)$raw), raio$data))
-msg <- n1$recv_aio(mode = "numer", keep.raw = TRUE, timeout = 500)
+nanotestaio(msg <- n1$recv_aio(mode = "numer", keep.raw = TRUE, timeout = 500))
 nanotest(identical(call_aio(msg), msg))
-msg <- n1$recv_aio(mode = "complex", keep.raw = TRUE, timeout = 500)
+nanotestaio(msg <- n1$recv_aio(mode = "complex", keep.raw = TRUE, timeout = 500))
 nanotestn(stop_aio(msg))
 nanotestn(stop_aio(n))
 nanotest(identical(call_aio(msg), msg))
@@ -105,37 +104,37 @@ nanotesterr(msg$raw, "active")
 nanotesterr(msg$data, "active")
 nanotest(identical(call_aio(n), n))
 nanotest(!is_aio(n))
-sraio <- n$send_aio(as.raw(0L), mode = "r", timeout = 500)
-rraio <- n1$recv_aio(mode = "r", timeout = 500)
-nanotestnn(call_aio(rraio)$data)
-sraio <- n$send_aio(c(1+2i, 4+3i), mode = "ra", timeout = 500)
-rraio <- n1$recv_aio(mode = "co", timeout = 500)
-nanotestnn(call_aio(rraio)$data)
-sraio <- n$send_aio(5, mode = "raw", timeout = 500)
-rraio <- n1$recv_aio(mode = "d", timeout = 500)
-nanotestnn(call_aio(rraio)$data)
-sraio <- n$send_aio(c(1, 2), mode = "raw", timeout = 500)
-rraio <- n1$recv_aio(mode = "n", timeout = 500)
-nanotestnn(call_aio(rraio)$data)
-sraio <- n$send_aio(c(1L, 2L, 3L), mode = "raw", timeout = 500)
-rraio <- n1$recv_aio(mode = "i", timeout = 500)
-nanotestnn(call_aio(rraio)$data)
-sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500)
-rraio <- n1$recv_aio(mode = "double", timeout = 500)
+nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "r", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "r", timeout = 500))
+nanotest(is.raw(call_aio(rraio)$data))
+nanotestaio(sraio <- n$send_aio(c(1+2i, 4+3i), mode = "ra", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "co", timeout = 500))
+nanotest(is.complex(call_aio(rraio)$data))
+nanotestaio(sraio <- n$send_aio(5, mode = "raw", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "d", timeout = 500))
+nanotest(is.double(call_aio(rraio)$data))
+nanotestaio(sraio <- n$send_aio(c(1, 2), mode = "raw", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "n", timeout = 500))
+nanotest(is.numeric(call_aio(rraio)$data))
+nanotestaio(sraio <- n$send_aio(c(1L, 2L, 3L), mode = "raw", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "i", timeout = 500))
+nanotest(is.integer(call_aio(rraio)$data))
+nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "double", timeout = 500))
 nanotestw(is.raw(call_aio(rraio)$data))
-sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500)
-rraio <- n1$recv_aio(mode = "int", timeout = 500)
+nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "int", timeout = 500))
 nanotestw(is.raw(call_aio(rraio)$data))
-sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500)
-rraio <- n1$recv_aio(mode = "logical", timeout = 500)
+nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "logical", timeout = 500))
 nanotestw(is.raw(call_aio(rraio)$data))
-sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500)
-rraio <- n1$recv_aio(mode = "numeric", timeout = 500)
+nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "numeric", timeout = 500))
 nanotestw(is.raw(call_aio(rraio)$data))
-sraio <- n$send_aio("hello", mode = 2L, timeout = 500)
+nanotestaio(sraio <- n$send_aio("hello", mode = 2L, timeout = 500))
 assign("aio", value = "test", envir = sraio)
 nanotesterr(sraio$result, "valid")
-rraio <- n1$recv_aio(mode = "characte", keep.raw = TRUE, timeout = 500)
+nanotestaio(rraio <- n1$recv_aio(mode = "characte", keep.raw = TRUE, timeout = 500))
 nanotesterr(opt(rraio[["aio"]], "false") <- 0L, "valid")
 nanotesterr(subscribe(rraio[["aio"]], "false"), "valid")
 nanotesterr(opt(rraio[["aio"]], "false"), "valid")
@@ -163,8 +162,8 @@ nanotestw(n1$close() == 7L)
 nanotest(n$socket[["state"]] == "closed")
 nanotest(n1$socket[state] == "closed")
 
-req <- nano("req", listen = "inproc://testing")
-rep <- socket("rep", dial = "inproc://testing", listen = "inproc://testing2")
+nanotestnano(req <- nano("req", listen = "inproc://testing"))
+nanotestnano(rep <- socket("rep", dial = "inproc://testing", listen = "inproc://testing2"))
 nanotestp(rep)
 if (nng_version()[1] == "1.6.0pre") {
   nanotest(stat(rep, "dialers") == 1)
@@ -188,30 +187,31 @@ nanotesterr(req$opt("false", "false"), "supported")
 nanotesterr(req$opt("false", NULL), "supported")
 nanotesterr(req$opt("false", TRUE), "supported")
 nanotesterr(req$opt("false", list()), "type")
-ctx <- context(rep)
+nanotestnano(ctx <- context(rep))
 nanotestp(ctx)
-csaio <- req$send_aio(data.frame(), mode = "seria", timeout = 500)
-nanotestnn(call_aio(csaio)$result)
-nanotestnn(recv_aio(ctx, timeout = 500))
-nanotestnn(req$send("context test", mode ="raw", block = FALSE))
-nanotestnn(recv(ctx, mode = "ch", block = FALSE))
+nanotestaio(csaio <- req$send_aio(data.frame(), mode = "seria", timeout = 500))
+nanotest(is.integer(call_aio(csaio)$result))
+nanotestaio(craio <- recv_aio(ctx, timeout = 500))
+nanotest(is.list(.subset2(call_aio(craio), "data")))
+nanotest(is.integer(req$send("context test", mode ="raw", block = 500)))
+nanotest(recv(ctx, mode = "ch", block = 500) == "context test")
 nanotestnn(req$send(data.frame(), mode = "seri", block = 500))
-msg <- recv_aio(ctx, mode = "ser", timeout = 500, keep.raw = FALSE)
+nanotestaio(msg <- recv_aio(ctx, mode = "ser", timeout = 500, keep.raw = FALSE))
 nanotest(is.logical(.unresolved(msg)))
 nanotest(is.logical(unresolved(msg)))
-nanotestnn(call_aio(msg)$data)
+nanotest(is.list(call_aio(msg)$data))
 nanotest(!unresolved(msg))
-nanotestnn(req$send(c(TRUE, FALSE, TRUE), mode = 2L, block = 500))
-msg <- recv_aio(ctx, mode = 6L, timeout = 500, keep.raw = TRUE)
-nanotestnn(call_aio(msg)$raw)
-nanotestnn(msg$raw)
-err <- send_aio(ctx, "test")
+nanotest(is.integer(req$send(c(TRUE, FALSE, TRUE), mode = 2L, block = 500)))
+nanotestaio(msg <- recv_aio(ctx, mode = 6L, timeout = 500, keep.raw = TRUE))
+nanotest(is.raw(call_aio(msg)$raw))
+nanotestaio(err <- send_aio(ctx, "test"))
 nanotestn(stop_aio(err))
 nanotesterr(err$result, "active")
-err <- send_aio(ctx, "test")
+nanotestaio(err <- send_aio(ctx, "test"))
 nanotest(is_error_value(call_aio(err)$result))
-nanotestnn(request(req$context, c(1+3i, 4+2i), recv_mode = "complex", timeout = 500, keep.raw = TRUE))
-nanotestnn(reply(ctx, execute = identity, send_mode = "ra", timeout = 500))
+nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), recv_mode = "complex", timeout = 500, keep.raw = TRUE))
+nanotest(is.integer(reply(ctx, execute = identity, send_mode = "ra", timeout = 500)))
+nanotest(is.complex(call_aio(rek)[["data"]]))
 
 nanotest(inherits(cv <- cv(), "conditionVariable"))
 nanotestp(cv)
@@ -219,10 +219,10 @@ nanotest(until(cv, 10L))
 nanotest(until(cv, 10))
 nanotestn(cv_reset(cv))
 nanotestz(cv_value(cv))
-nanotestnn(cr <- request_signal(req$context, "test", timeout = 500, keep.raw = TRUE, cv = cv))
-nanotestnn(cr <- recv_aio_signal(rep, timeout = 500, keep.raw = TRUE, cv = cv))
-nanotestnn(call_aio(cr)$raw)
-nanotestnn(cr <- recv_aio_signal(context(rep), timeout = 500, cv = cv))
+nanotestaio(cr <- request_signal(req$context, "test", timeout = 500, keep.raw = TRUE, cv = cv))
+nanotestaio(cr <- recv_aio_signal(rep, timeout = 500, keep.raw = TRUE, cv = cv))
+nanotest(is.raw(call_aio(cr)$raw))
+nanotestaio(cr <- recv_aio_signal(context(rep), timeout = 500, cv = cv))
 nanotestz(pipe_notify(rep, cv))
 nanotestz(pipe_notify(req$socket, cv = cv, add = FALSE, remove = TRUE, flag = FALSE))
 nanotesterr(request_signal(err, "test", cv = cv), "valid")
@@ -234,7 +234,6 @@ nanotesterr(until(err, 10), "valid")
 nanotesterr(cv_value(err), "valid")
 nanotesterr(cv_reset(err), "valid")
 nanotesterr(cv_signal(err), "valid")
-nanotesterr(timed_signal(err, 1000), "valid")
 nanotesterr(pipe_notify(err, cv), "valid Socket")
 nanotesterr(pipe_notify(rep, err), "valid Condition Variable")
 nanotestz(req$context_close())
@@ -246,10 +245,10 @@ nanotestxp(rep$dialer[[1L]])
 nanotestz(close(ctx))
 nanotestz(close(rep))
 
-pub <- nano("pub", listen = "inproc://ps")
-sub <- nano("sub", dial = "inproc://ps", autostart = NA)
-cv <- cv()
-cv2 <- cv()
+nanotestnano(pub <- nano("pub", listen = "inproc://ps"))
+nanotestnano(sub <- nano("sub", dial = "inproc://ps", autostart = NA))
+nanotestxp(cv <- cv())
+nanotestxp(cv2 <- cv())
 nanotestz(pipe_notify(pub$socket, cv, cv2, flag = FALSE))
 nanotestnano(sub$opt(name = "sub:prefnew", value = FALSE))
 nanotest(!sub$opt(name = "sub:prefnew"))
@@ -270,17 +269,12 @@ nanotestz(sub$close())
 nanotestz(pub$close())
 nanotest(wait(cv))
 nanotest(wait(cv2))
-nanotestxp(sig <- timed_signal(cv, time = 50L))
-nanotest(!wait(cv))
-nanotestxp(sig <- timed_signal(cv2, time = 50, flag = FALSE))
-nanotest(wait(cv2))
-nanotestp(sig)
 nanotestn(cv_signal(cv))
 nanotest(cv_value(cv) == 1L)
 
-surv <- nano(protocol = "surveyor", listen = "inproc://sock1", dial = "inproc://sock2")
+nanotestnano(surv <- nano(protocol = "surveyor", listen = "inproc://sock1", dial = "inproc://sock2"))
 nanotestp(surv)
-resp <- nano(protocol = "respondent", listen = "inproc://sock2", dial = "inproc://sock1")
+nanotestnano(resp <- nano(protocol = "respondent", listen = "inproc://sock2", dial = "inproc://sock1"))
 nanotestz(pipe_notify(surv$socket, cv, cv2, flag = TRUE))
 surv$dialer <- NULL
 nanotestxp(surv$dialer[[1L]])
@@ -320,14 +314,14 @@ nanotestnn(ncurl("http://www.cam.ac.uk/", follow = FALSE, response = list("date"
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = TRUE))
 nanotestnn(ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L))
 nanotestnn(ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000))
-relo <- ncurl("https://cran.r-project.org/package=nanonext", convert = FALSE, response = "server", async = TRUE)
+relo <- ncurl("https://trin.cam.ac.uk/", convert = FALSE, response = "server", async = TRUE)
 nanotest(is.integer(call_aio(relo)$status))
-relo <- ncurl("https://cran.r-project.org/package=nanonext", convert = FALSE, response = list("server"), async = TRUE)
+relo <- ncurl("https://trin.cam.ac.uk/", convert = FALSE, response = list("server"), async = TRUE)
 nanotest(is.integer(call_aio(relo)$status))
 put1 <- ncurl("http://postman-echo.com/put", async = TRUE, method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", response = c("date", "server"), timeout = 3000L)
 nanotest(is.integer(call_aio(put1)$status))
 nanotestnn(put1$headers)
-nanotestnn(put1$raw)
+nanotest(is.null(put1$raw) || is_error_value(put1$raw))
 nanotestnn(put1$data)
 put2 <- ncurl("http://postman-echo.com/put", async = TRUE, method = "PUT", headers = list(Authorization = "Bearer token"), response = list("date", "server"), timeout = 3000L)
 nanotestp(put2)
@@ -364,6 +358,7 @@ nanotestn(msleep("a"))
 nanotest(is.character(urlp <- parse_url("tcp://192.168.0.2:5555")) && length(urlp) == 10L)
 nanotest(is.double(random()))
 nanotest(length(random(2)) == 2L)
+nanotest(length(random(3L)) == 3L)
 nanotesterr(parse_url("tcp:/"), "argument")
 nanotesterr(random("test"), "integer")
 nanotesterr(device(push, pull), "closed")
@@ -371,6 +366,7 @@ nanotesterr(device(ctx, push), "s1")
 nanotesterr(device(push, ctx), "s2")
 for (i in c(100:103, 200:208, 226, 300:308, 400:426, 428:431, 451, 500:511))
   nanotest(is.character(status_code(i)))
+nanotest(strcat("hello ", "world") == "hello world")
 
 s <- tryCatch(stream(dial = "wss://echo.websocket.events/", textframes = TRUE), error = function(e) NULL)
 if (length(s)) {
@@ -380,15 +376,15 @@ if (length(s)) {
   nanotest(is.integer(send(s, "message2", block = FALSE)))
   nanotestnn(recv(s, mode = 2L, block = 100))
   nanotest(is.integer(send(s, 2L, block = 500)))
-  nanotest(is_aio(sr <- recv_aio(s, mode = "integer", keep.raw = TRUE, timeout = 500L, n = 8192L)))
-  nanotestnn(call_aio(sr)[["raw"]])
+  nanotestaio(sr <- recv_aio(s, mode = "integer", keep.raw = TRUE, timeout = 500L, n = 8192L))
+  nanotestw(length(call_aio(sr)[["raw"]]))
   nanotestn(stop_aio(sr))
-  nanotest(is_aio(ss <- send_aio(s, "async", timeout = 500L)))
+  nanotestaio(ss <- send_aio(s, "async", timeout = 500L))
   nanotest(is.integer(call_aio(ss)[["result"]]))
   nanotestn(stop_aio(ss))
   nanotest(is.integer(send(s, 12.56, mode = "raw", block = 500L)))
-  nanotest(is_aio(sr <- recv_aio_signal(s, mode = "double", timeout = 500L, cv = cv)))
-  nanotestnn(call_aio(sr)[["data"]])
+  nanotestaio(sr <- recv_aio_signal(s, mode = "double", timeout = 500L, cv = cv))
+  nanotestw(length(call_aio(sr)[["data"]]))
   nanotest(cv_value(cv) > 0L)
   nanotest(!wait(cv))
   nanotest(is.character(opt(s, "ws:request-headers")))
@@ -400,25 +396,25 @@ if (length(s)) {
   nanotest(is.integer(close(s)))
 }
 
-s <- socket("bus", listen = "inproc://nanolock")
-s1 <- socket("bus", dial = "inproc://nanolock")
+nanotestnano(s <- socket("bus", listen = "inproc://nanolock"))
+nanotestnano(s1 <- socket("bus", dial = "inproc://nanolock"))
 nanotestz(lock(s))
-s2 <- socket("bus", dial = "inproc://nanolock")
-nanotestz(send(s, "test"))
-nanotestnn(recv(s1))
+nanotestnano(s2 <- socket("bus", dial = "inproc://nanolock"))
+nanotestaio(send_aio(s, "test"))
+nanotestnn(recv(s1, block = 500))
 nanotest(is_error_value(recv(s2)))
 nanotestz(unlock(s))
 nanotesterr(unlock(cv), "valid Socket")
-cv <- cv()
+nanotestxp(cv <- cv())
 nanotestz(pipe_notify(s, cv = cv, flag = FALSE))
-s3 <- socket("bus", dial = "inproc://nanolock")
-nanotestz(send(s, "test"))
-nanotestnn(recv(s3))
+nanotestnano(s3 <- socket("bus", dial = "inproc://nanolock"))
+nanotestz(send(s, "test", block = 500))
+nanotestnn(recv(s3, block = 500))
 nanotestz(lock(s, cv = cv))
 nanotesterr(lock(cv), "valid Socket")
 nanotesterr(lock(s, cv = s), "valid Condition Variable")
-s4 <- socket("bus", dial = "inproc://nanolock")
-nanotestz(send(s, "test"))
+nanotestnano(s4 <- socket("bus", dial = "inproc://nanolock"))
+nanotestaio(send_aio(s, "test"))
 nanotest(is_error_value(recv(s4)))
 nanotestz(close(s))
 nanotestz(close(s1))
@@ -429,7 +425,7 @@ nanotestz(close(s4))
 nanotest(nanonext:::.DollarNames.ncurlAio(NULL, "sta") == "status")
 nanotest(nanonext:::.DollarNames.recvAio(NULL, "dat") == "data")
 nanotest(nanonext:::.DollarNames.sendAio(NULL, "r") == "result")
-nanotestnn(nanonext:::.DollarNames.nano(NULL))
+nanotest(length(nanonext:::.DollarNames.nano(NULL)) == 0L)
 
 fakesock <- `class<-`(new.env(), "nanoSocket")
 nanotesterr(dial(fakesock), "valid Socket")
@@ -440,9 +436,9 @@ nanotesterr(context(fakesock), "valid Socket")
 nanotesterr(.context(fakesock), "valid Socket")
 nanotesterr(stat(fakesock, "pipes"), "valid Socket")
 nanotesterr(close(fakesock), "valid Socket")
-nanotest(.unresolved(fakesock) == FALSE)
+nanotest(!.unresolved(fakesock))
 fakectx <- `class<-`("test", "nanoContext")
-nanotest(.unresolved(fakectx) == FALSE)
+nanotest(!.unresolved(fakectx))
 nanotesterr(request(fakectx, data = "test"), "valid Context")
 nanotesterr(subscribe(fakectx, NULL), "valid")
 nanotesterr(close(fakectx), "valid Context")
@@ -467,7 +463,7 @@ nanotesterr(start(fakelist), "valid Listener")
 nanotesterr(close(fakelist), "valid Listener")
 unres <- `class<-`(NA, "unresolvedValue")
 nanotestp(unres)
-nanotest(unresolved(unres))
+nanotest(!unresolved(unres))
 nanotest(identical(call_aio("a"), "a"))
 nanotestn(stop_aio("a"))
 
@@ -475,7 +471,7 @@ nanotest(sha256("test") == "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d
 nanotest(length(sha256(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) == 32L)
 nanotest(nchar(sha512(c(1.1, 2.2), as.raw(1L))) == 128L)
 nanotest(length(sha512("testing", convert = FALSE)) == 64L)
-nanotest(length(sha384(data.frame(), "KEY", convert = FALSE)) == 48L)
+nanotest(length(sha384(as.list(random(1e4L)), "KEY", convert = FALSE)) == 48L)
 nanotest(nchar(sha384(7000L, key = NULL, convert = TRUE)) == 96L)
 nanotest(length(sha224(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) == 28L)
 nanotest(sha224(as.raw(0L)) == "fff9292b4201617bdc4d3053fce02734166a683d7d858a7f5f59b073")
@@ -486,9 +482,18 @@ nanotest(is.character(sha256(quote(a))))
 nanotest(base64enc("test") == "dGVzdA==")
 nanotest(base64dec(base64enc("test")) == "test")
 nanotest(is.raw(base64enc(data.frame(), convert = FALSE)))
-nanotest(is.integer(unserialize(base64dec(base64enc(c(1L, 2L)), convert = FALSE))))
+nanotest(is.raw(base64dec(base64enc(as.raw(c(1L, 2L)), convert = FALSE), convert = FALSE)))
+nanotest(is.integer(base64dec(base64enc(c(1L, 2L)), convert = NA)))
 nanotesterr(base64dec("__"), "not valid base64")
 
+k <- new.env()
+v <- "value"
+nanotest(typeof(w <- weakref(k, v)) == "weakref")
+nanotest(identical(weakref_key(w), k))
+nanotest(identical(weakref_value(w), v))
+rm(v)
+nanotest(weakref_value(w) == "value")
+
 file <- tempfile()
 pem <- "-----BEGIN CERTIFICATE----- -----END CERTIFICATE-----"
 cat(pem, file = file)
@@ -497,27 +502,20 @@ nanotesterr(tls_config(server = file), "Cryptographic error")
 nanotesterr(tls_config(client = c(pem, pem)), "Cryptographic error")
 nanotesterr(tls_config(server = c(pem, pem)), "Cryptographic error")
 unlink(file)
-cert <- write_cert(cn = "127.0.0.1")
-nanotest(is.list(cert) && length(cert) == 2L)
+nanotest(is.list(cert <- write_cert(cn = "127.0.0.1")))
+nanotest(length(cert) == 2L && is.character(cert[[1L]]))
 nanotest(identical(names(cert), c("server", "client")))
 nanotestxp(tls <- tls_config(client = cert$client))
 nanotest(inherits(tls, "tlsConfig"))
 nanotestp(tls)
-nanotest(is_error_value(ncurl("https://www.r-project.org/", tls = tls)$status))
-nanotest(is_error_value(call_aio(ncurl("https://www.r-project.org/", async = TRUE, tls = tls))$status))
+nanotest(is_error_value(ncurl("https://www.cam.ac.uk/", tls = tls)$status))
+nanotest(is_error_value(call_aio(ncurl("https://www.cam.ac.uk/", async = TRUE, tls = tls))$status))
 nanotestxp(s <- socket(listen = "tls+tcp://127.0.0.1:5556", tls = tls_config(server = cert$server)))
 nanotestxp(s1 <- socket(dial = "tls+tcp://127.0.0.1:5556", tls = tls))
 nanotestw(dial(s, url = "tls+tcp://[", tls = tls, error = FALSE) > 0)
 nanotestw(listen(s, url = "tls+tcp://[", tls = tls, error = FALSE) > 0)
-close(s1)
-close(s)
+nanotestz(close(s1))
+nanotestz(close(s))
 nanotestxp(tls <- tls_config())
 nanotestxp(s <- socket(listen = "wss://127.0.0.1:5557", tls = tls))
 nanotestz(close(s))
-
-k <- new.env()
-v <- "test"
-nanotest(typeof(w <- weakref(k, v)) == "weakref")
-nanotest(identical(weakref_key(w), k))
-nanotest(identical(weakref_value(w), v))
-nanotest(strcat("hello ", "world") == "hello world")
